@@ -132,7 +132,10 @@ _op_str_mapping = {
     roperator.rtruediv: "/",
     operator.floordiv: "//",
     roperator.rfloordiv: "//",
-    operator.mod: "%",
+    # we require Python semantics for mod of negative for backwards compatibility
+    # see https://github.com/pydata/numexpr/issues/365
+    # so sticking with unaccelerated for now
+    operator.mod: None,
     roperator.rmod: "%",
     operator.pow: "**",
     roperator.rpow: "**",
