@@ -237,7 +237,6 @@ class Grouper:
             #   core/groupby/grouper.py::Grouper
             # raising these warnings from TimeGrouper directly would fail the test:
             #   tests/resample/test_deprecated.py::test_deprecating_on_loffset_and_base
-
             # hacky way to set the stacklevel: if cls is TimeGrouper it means
             # that the call comes from a pandas internal call of resample,
             # otherwise it comes from pd.Grouper
@@ -557,8 +556,13 @@ class Grouping:
         if isinstance(self.grouper, ops.BaseGrouper):
             return self.grouper.indices
 
-        values = Categorical(self.grouper)
-        return values._reverse_indexer()
+        # Return a dictionary of {group label: [indices belonging to the group label]}
+        # respecting whether sort was specified
+        codes, uniques = algorithms.factorize(self.grouper, sort=self.sort)
+        return {
+            category: np.flatnonzero(codes == i)
+            for i, category in enumerate(Index(uniques))
+        }
 
     @property
     def codes(self) -> np.ndarray:
@@ -586,8 +590,13 @@ class Grouping:
                 codes = self.grouper.codes_info
                 uniques = self.grouper.result_index
             else:
+                # GH35667, replace dropna=False with na_sentinel=None
+                if not self.dropna:
+                    na_sentinel = None
+                else:
+                    na_sentinel = -1
                 codes, uniques = algorithms.factorize(
-                    self.grouper, sort=self.sort, dropna=self.dropna
+                    self.grouper, sort=self.sort, na_sentinel=na_sentinel
                 )
                 uniques = Index(uniques, name=self.name)
             self._codes = codes
