@@ -135,13 +135,20 @@ class TestDataFrameMissingData:
         df2 = df.copy()
         df["A"].dropna()
         tm.assert_series_equal(df["A"], original)
-        return_value = df["A"].dropna(inplace=True)
-        tm.assert_series_equal(df["A"], expected)
+
+        ser = df["A"]
+        return_value = ser.dropna(inplace=True)
+        tm.assert_series_equal(ser, expected)
+        tm.assert_series_equal(df["A"], original)
         assert return_value is None
+
         df2["A"].drop([1])
         tm.assert_series_equal(df2["A"], original)
-        return_value = df2["A"].drop([1], inplace=True)
-        tm.assert_series_equal(df2["A"], original.drop([1]))
+
+        ser = df2["A"]
+        return_value = ser.drop([1], inplace=True)
+        tm.assert_series_equal(ser, original.drop([1]))
+        tm.assert_series_equal(df2["A"], original)
         assert return_value is None
 
     def test_dropna_corner(self, float_frame):
