@@ -8,7 +8,7 @@ import numpy as np
 import pytest
 
 import pandas as pd
-from pandas import NaT, Timedelta, Timestamp, _is_numpy_dev, offsets
+from pandas import NaT, Timedelta, Timestamp, _is_numpy_dev, compat, offsets
 import pandas._testing as tm
 from pandas.core import ops
 
@@ -88,6 +88,14 @@ class TestTimedeltaAdditionSubtraction:
         result = op(td, NaT)
         assert result is NaT
 
+    def test_td_add_timestamp_overflow(self):
+        msg = "int too (large|big) to convert"
+        with pytest.raises(OverflowError, match=msg):
+            Timestamp("1700-01-01") + Timedelta(13 * 19999, unit="D")
+
+        with pytest.raises(OverflowError, match=msg):
+            Timestamp("1700-01-01") + timedelta(days=13 * 19999)
+
     @pytest.mark.parametrize("op", [operator.add, ops.radd])
     def test_td_add_td(self, op):
         td = Timedelta(10, unit="d")
@@ -173,14 +181,15 @@ class TestTimedeltaAdditionSubtraction:
 
     def test_td_add_sub_numeric_raises(self):
         td = Timedelta(10, unit="d")
+        msg = "unsupported operand type"
         for other in [2, 2.0, np.int64(2), np.float64(2)]:
-            with pytest.raises(TypeError):
+            with pytest.raises(TypeError, match=msg):
                 td + other
-            with pytest.raises(TypeError):
+            with pytest.raises(TypeError, match=msg):
                 other + td
-            with pytest.raises(TypeError):
+            with pytest.raises(TypeError, match=msg):
                 td - other
-            with pytest.raises(TypeError):
+            with pytest.raises(TypeError, match=msg):
                 other - td
 
     def test_td_rsub_nat(self):
@@ -221,7 +230,8 @@ class TestTimedeltaAdditionSubtraction:
         # GH#21980
         now = Timestamp.now()
         arr = np.array([now, Timedelta("1D"), np.timedelta64(2, "h")])
-        with pytest.raises(TypeError):
+        msg = r"unsupported operand type\(s\) for \-: 'Timedelta' and 'Timestamp'"
+        with pytest.raises(TypeError, match=msg):
             Timedelta("1D") - arr
 
     @pytest.mark.parametrize("op", [operator.add, ops.radd])
@@ -253,7 +263,10 @@ class TestTimedeltaAdditionSubtraction:
         msg = r"unsupported operand type\(s\) for \+: 'Timedelta' and 'int'"
         with pytest.raises(TypeError, match=msg):
             td + np.array([1])
-        msg = r"unsupported operand type\(s\) for \+: 'numpy.ndarray' and 'Timedelta'"
+        msg = (
+            r"unsupported operand type\(s\) for \+: 'numpy.ndarray' and 'Timedelta'|"
+            "Concatenation operation is not implemented for NumPy arrays"
+        )
         with pytest.raises(TypeError, match=msg):
             np.array([1]) + td
 
@@ -271,8 +284,8 @@ class TestTimedeltaAdditionSubtraction:
         tm.assert_numpy_array_equal(td * np.array([2]), expected)
         tm.assert_numpy_array_equal(np.array([2]) * td, expected)
         msg = (
-            "ufunc '?multiply'? cannot use operands with types"
-            r" dtype\('<m8\[ns\]'\) and dtype\('<m8\[ns\]'\)"
+            "ufunc '?multiply'? cannot use operands with types "
+            r"dtype\('<m8\[ns\]'\) and dtype\('<m8\[ns\]'\)"
         )
         with pytest.raises(TypeError, match=msg):
             td * other
@@ -315,7 +328,14 @@ class TestTimedeltaMultiplicationDivision:
     def test_td_mul_nat(self, op, td_nat):
         # GH#19819
         td = Timedelta(10, unit="d")
-        with pytest.raises(TypeError):
+        typs = "|".join(["numpy.timedelta64", "NaTType", "Timedelta"])
+        msg = "|".join(
+            [
+                rf"unsupported operand type\(s\) for \*: '{typs}' and '{typs}'",
+                r"ufunc '?multiply'? cannot use operands with types",
+            ]
+        )
+        with pytest.raises(TypeError, match=msg):
             op(td, td_nat)
 
     @pytest.mark.parametrize("nan", [np.nan, np.float64("NaN"), float("nan")])
@@ -342,11 +362,12 @@ class TestTimedeltaMultiplicationDivision:
         assert op(-1, td).value == -1 * td.value
         assert op(-1.0, td).value == -1.0 * td.value
 
-        with pytest.raises(TypeError):
+        msg = "unsupported operand type"
+        with pytest.raises(TypeError, match=msg):
             # timedelta * datetime is gibberish
             op(td, Timestamp(2016, 1, 2))
 
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError, match=msg):
             # invalid multiply with another timedelta
             op(td, td)
 
@@ -365,6 +386,26 @@ class TestTimedeltaMultiplicationDivision:
 
         assert np.isnan(td / NaT)
 
+    def test_td_div_td64_non_nano(self):
+
+        # truediv
+        td = Timedelta("1 days 2 hours 3 ns")
+        result = td / np.timedelta64(1, "D")
+        assert result == td.value / float(86400 * 1e9)
+        result = td / np.timedelta64(1, "s")
+        assert result == td.value / float(1e9)
+        result = td / np.timedelta64(1, "ns")
+        assert result == td.value
+
+        # floordiv
+        td = Timedelta("1 days 2 hours 3 ns")
+        result = td // np.timedelta64(1, "D")
+        assert result == 1
+        result = td // np.timedelta64(1, "s")
+        assert result == 93600
+        result = td // np.timedelta64(1, "ns")
+        assert result == td.value
+
     def test_td_div_numeric_scalar(self):
         # GH#19738
         td = Timedelta(10, unit="d")
@@ -384,9 +425,10 @@ class TestTimedeltaMultiplicationDivision:
             pytest.param(
                 np.float64("NaN"),
                 marks=pytest.mark.xfail(
-                    _is_numpy_dev,
+                    # Works on numpy dev only in python 3.9
+                    _is_numpy_dev and not compat.PY39,
+                    raises=RuntimeWarning,
                     reason="https://github.com/pandas-dev/pandas/issues/31992",
-                    strict=False,
                 ),
             ),
             float("nan"),
@@ -412,6 +454,50 @@ class TestTimedeltaMultiplicationDivision:
 
         assert np.timedelta64(60, "h") / td == 0.25
 
+    def test_td_rdiv_na_scalar(self):
+        # GH#31869 None gets cast to NaT
+        td = Timedelta(10, unit="d")
+
+        result = NaT / td
+        assert np.isnan(result)
+
+        result = None / td
+        assert np.isnan(result)
+
+        result = np.timedelta64("NaT") / td
+        assert np.isnan(result)
+
+        msg = r"unsupported operand type\(s\) for /: 'numpy.datetime64' and 'Timedelta'"
+        with pytest.raises(TypeError, match=msg):
+            np.datetime64("NaT") / td
+
+        msg = r"unsupported operand type\(s\) for /: 'float' and 'Timedelta'"
+        with pytest.raises(TypeError, match=msg):
+            np.nan / td
+
+    def test_td_rdiv_ndarray(self):
+        td = Timedelta(10, unit="d")
+
+        arr = np.array([td], dtype=object)
+        result = arr / td
+        expected = np.array([1], dtype=np.float64)
+        tm.assert_numpy_array_equal(result, expected)
+
+        arr = np.array([None])
+        result = arr / td
+        expected = np.array([np.nan])
+        tm.assert_numpy_array_equal(result, expected)
+
+        arr = np.array([np.nan], dtype=object)
+        msg = r"unsupported operand type\(s\) for /: 'float' and 'Timedelta'"
+        with pytest.raises(TypeError, match=msg):
+            arr / td
+
+        arr = np.array([np.nan], dtype=np.float64)
+        msg = "cannot use operands with types dtype"
+        with pytest.raises(TypeError, match=msg):
+            arr / td
+
     # ---------------------------------------------------------------
     # Timedelta.__floordiv__
 
@@ -442,7 +528,14 @@ class TestTimedeltaMultiplicationDivision:
         # GH#18846
         td = Timedelta(hours=3, minutes=4)
 
-        with pytest.raises(TypeError):
+        msg = "|".join(
+            [
+                r"Invalid dtype datetime64\[D\] for __floordiv__",
+                "'dtype' is an invalid keyword argument for this function",
+                r"ufunc '?floor_divide'? cannot use operands with types",
+            ]
+        )
+        with pytest.raises(TypeError, match=msg):
             td // np.datetime64("2016-01-01", dtype="datetime64[us]")
 
     def test_td_floordiv_numeric_scalar(self):
@@ -513,8 +606,14 @@ class TestTimedeltaMultiplicationDivision:
         td = Timedelta(hours=3, minutes=3)
 
         dt64 = np.datetime64("2016-01-01", "us")
-        with pytest.raises(TypeError):
-            td.__rfloordiv__(dt64)
+
+        assert td.__rfloordiv__(dt64) is NotImplemented
+
+        msg = (
+            r"unsupported operand type\(s\) for //: 'numpy.datetime64' and 'Timedelta'"
+        )
+        with pytest.raises(TypeError, match=msg):
+            dt64 // td
 
     def test_td_rfloordiv_numeric_scalar(self):
         # GH#18846
@@ -523,14 +622,18 @@ class TestTimedeltaMultiplicationDivision:
         assert td.__rfloordiv__(np.nan) is NotImplemented
         assert td.__rfloordiv__(3.5) is NotImplemented
         assert td.__rfloordiv__(2) is NotImplemented
+        assert td.__rfloordiv__(np.float64(2.0)) is NotImplemented
+        assert td.__rfloordiv__(np.uint8(9)) is NotImplemented
+        assert td.__rfloordiv__(np.int32(2.0)) is NotImplemented
 
-        with pytest.raises(TypeError):
-            td.__rfloordiv__(np.float64(2.0))
-        with pytest.raises(TypeError):
-            td.__rfloordiv__(np.uint8(9))
-        with pytest.raises(TypeError, match="Invalid dtype"):
+        msg = r"unsupported operand type\(s\) for //: '.*' and 'Timedelta"
+        with pytest.raises(TypeError, match=msg):
+            np.float64(2.0) // td
+        with pytest.raises(TypeError, match=msg):
+            np.uint8(9) // td
+        with pytest.raises(TypeError, match=msg):
             # deprecated GH#19761, enforced GH#29797
-            td.__rfloordiv__(np.int32(2.0))
+            np.int32(2.0) // td
 
     def test_td_rfloordiv_timedeltalike_array(self):
         # GH#18846
@@ -549,6 +652,14 @@ class TestTimedeltaMultiplicationDivision:
         expected = np.array([10, np.nan])
         tm.assert_numpy_array_equal(res, expected)
 
+    def test_td_rfloordiv_intarray(self):
+        # deprecated GH#19761, enforced GH#29797
+        ints = np.array([1349654400, 1349740800, 1349827200, 1349913600]) * 10 ** 9
+
+        msg = "Invalid dtype"
+        with pytest.raises(TypeError, match=msg):
+            ints // Timedelta(1, unit="s")
+
     def test_td_rfloordiv_numeric_series(self):
         # GH#18846
         td = Timedelta(hours=3, minutes=3)
@@ -556,9 +667,9 @@ class TestTimedeltaMultiplicationDivision:
         res = td.__rfloordiv__(ser)
         assert res is NotImplemented
 
-        with pytest.raises(TypeError, match="Invalid dtype"):
+        msg = "Invalid dtype"
+        with pytest.raises(TypeError, match=msg):
             # Deprecated GH#19761, enforced GH#29797
-            # TODO: GH-19761. Change to TypeError.
             ser // td
 
     # ----------------------------------------------------------------
@@ -623,11 +734,11 @@ class TestTimedeltaMultiplicationDivision:
     def test_mod_invalid(self):
         # GH#19365
         td = Timedelta(hours=37)
-
-        with pytest.raises(TypeError):
+        msg = "unsupported operand type"
+        with pytest.raises(TypeError, match=msg):
             td % Timestamp("2018-01-22")
 
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError, match=msg):
             td % []
 
     def test_rmod_pytimedelta(self):
@@ -649,16 +760,18 @@ class TestTimedeltaMultiplicationDivision:
         # GH#19365
         td = Timedelta(minutes=3)
 
-        with pytest.raises(TypeError):
+        msg = "unsupported operand"
+        with pytest.raises(TypeError, match=msg):
             Timestamp("2018-01-22") % td
 
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError, match=msg):
             15 % td
 
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError, match=msg):
             16.0 % td
 
-        with pytest.raises(TypeError):
+        msg = "Invalid dtype int"
+        with pytest.raises(TypeError, match=msg):
             np.array([22, 24]) % td
 
     # ----------------------------------------------------------------
@@ -709,7 +822,8 @@ class TestTimedeltaMultiplicationDivision:
         # GH#19365
         td = Timedelta(days=2, hours=6)
 
-        with pytest.raises(TypeError):
+        msg = r"unsupported operand type\(s\) for //: 'Timedelta' and 'Timestamp'"
+        with pytest.raises(TypeError, match=msg):
             divmod(td, Timestamp("2018-01-22"))
 
     def test_rdivmod_pytimedelta(self):
@@ -728,17 +842,19 @@ class TestTimedeltaMultiplicationDivision:
     def test_rdivmod_invalid(self):
         # GH#19365
         td = Timedelta(minutes=3)
+        msg = "unsupported operand type"
 
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError, match=msg):
             divmod(Timestamp("2018-01-22"), td)
 
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError, match=msg):
             divmod(15, td)
 
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError, match=msg):
             divmod(16.0, td)
 
-        with pytest.raises(TypeError):
+        msg = "Invalid dtype int"
+        with pytest.raises(TypeError, match=msg):
             divmod(np.array([22, 24]), td)
 
     # ----------------------------------------------------------------
@@ -754,5 +870,160 @@ class TestTimedeltaMultiplicationDivision:
         ],
     )
     def test_td_op_timedelta_timedeltalike_array(self, op, arr):
-        with pytest.raises(TypeError):
+        msg = "unsupported operand type|cannot use operands with types"
+        with pytest.raises(TypeError, match=msg):
             op(arr, Timedelta("1D"))
+
+
+class TestTimedeltaComparison:
+    def test_compare_tick(self, tick_classes):
+        cls = tick_classes
+
+        off = cls(4)
+        td = off.delta
+        assert isinstance(td, Timedelta)
+
+        assert td == off
+        assert not td != off
+        assert td <= off
+        assert td >= off
+        assert not td < off
+        assert not td > off
+
+        assert not td == 2 * off
+        assert td != 2 * off
+        assert td <= 2 * off
+        assert td < 2 * off
+        assert not td >= 2 * off
+        assert not td > 2 * off
+
+    def test_comparison_object_array(self):
+        # analogous to GH#15183
+        td = Timedelta("2 days")
+        other = Timedelta("3 hours")
+
+        arr = np.array([other, td], dtype=object)
+        res = arr == td
+        expected = np.array([False, True], dtype=bool)
+        assert (res == expected).all()
+
+        # 2D case
+        arr = np.array([[other, td], [td, other]], dtype=object)
+        res = arr != td
+        expected = np.array([[True, False], [False, True]], dtype=bool)
+        assert res.shape == expected.shape
+        assert (res == expected).all()
+
+    def test_compare_timedelta_ndarray(self):
+        # GH#11835
+        periods = [Timedelta("0 days 01:00:00"), Timedelta("0 days 01:00:00")]
+        arr = np.array(periods)
+        result = arr[0] > arr
+        expected = np.array([False, False])
+        tm.assert_numpy_array_equal(result, expected)
+
+    def test_compare_td64_ndarray(self):
+        # GG#33441
+        arr = np.arange(5).astype("timedelta64[ns]")
+        td = pd.Timedelta(arr[1])
+
+        expected = np.array([False, True, False, False, False], dtype=bool)
+
+        result = td == arr
+        tm.assert_numpy_array_equal(result, expected)
+
+        result = arr == td
+        tm.assert_numpy_array_equal(result, expected)
+
+        result = td != arr
+        tm.assert_numpy_array_equal(result, ~expected)
+
+        result = arr != td
+        tm.assert_numpy_array_equal(result, ~expected)
+
+    @pytest.mark.skip(reason="GH#20829 is reverted until after 0.24.0")
+    def test_compare_custom_object(self):
+        """
+        Make sure non supported operations on Timedelta returns NonImplemented
+        and yields to other operand (GH#20829).
+        """
+
+        class CustomClass:
+            def __init__(self, cmp_result=None):
+                self.cmp_result = cmp_result
+
+            def generic_result(self):
+                if self.cmp_result is None:
+                    return NotImplemented
+                else:
+                    return self.cmp_result
+
+            def __eq__(self, other):
+                return self.generic_result()
+
+            def __gt__(self, other):
+                return self.generic_result()
+
+        t = Timedelta("1s")
+
+        assert not (t == "string")
+        assert not (t == 1)
+        assert not (t == CustomClass())
+        assert not (t == CustomClass(cmp_result=False))
+
+        assert t < CustomClass(cmp_result=True)
+        assert not (t < CustomClass(cmp_result=False))
+
+        assert t == CustomClass(cmp_result=True)
+
+    @pytest.mark.parametrize("val", ["string", 1])
+    def test_compare_unknown_type(self, val):
+        # GH#20829
+        t = Timedelta("1s")
+        msg = "not supported between instances of 'Timedelta' and '(int|str)'"
+        with pytest.raises(TypeError, match=msg):
+            t >= val
+        with pytest.raises(TypeError, match=msg):
+            t > val
+        with pytest.raises(TypeError, match=msg):
+            t <= val
+        with pytest.raises(TypeError, match=msg):
+            t < val
+
+
+def test_ops_notimplemented():
+    class Other:
+        pass
+
+    other = Other()
+
+    td = Timedelta("1 day")
+    assert td.__add__(other) is NotImplemented
+    assert td.__sub__(other) is NotImplemented
+    assert td.__truediv__(other) is NotImplemented
+    assert td.__mul__(other) is NotImplemented
+    assert td.__floordiv__(other) is NotImplemented
+
+
+def test_ops_error_str():
+    # GH#13624
+    td = Timedelta("1 day")
+
+    for left, right in [(td, "a"), ("a", td)]:
+
+        msg = "|".join(
+            [
+                "unsupported operand type",
+                r'can only concatenate str \(not "Timedelta"\) to str',
+                "must be str, not Timedelta",
+            ]
+        )
+        with pytest.raises(TypeError, match=msg):
+            left + right
+
+        msg = "not supported between instances of"
+        with pytest.raises(TypeError, match=msg):
+            left > right
+
+        assert not left == right
+        assert left != right
