@@ -1,6 +1,7 @@
 import numpy as np
 import pytest
 
+from pandas import array
 import pandas._testing as tm
 from pandas.core.arrays.sparse import SparseArray
 
@@ -102,3 +103,11 @@ def test_assert_extension_array_equal_non_extension_array(side):
 
     with pytest.raises(AssertionError, match=msg):
         tm.assert_extension_array_equal(*args)
+
+
+@pytest.mark.parametrize("right_dtype", ["Int32", "int64"])
+def test_assert_extension_array_equal_ignore_dtype_mismatch(right_dtype):
+    # https://github.com/pandas-dev/pandas/issues/35715
+    left = array([1, 2, 3], dtype="Int64")
+    right = array([1, 2, 3], dtype=right_dtype)
+    tm.assert_extension_array_equal(left, right, check_dtype=False)
